package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.CapabilityItemHandler;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.BlockCargoCrate;
import ru.dbotthepony.mc.otm.container.MatteryContainer;
import ru.dbotthepony.mc.otm.container.MatteryContainerHandler;
import ru.dbotthepony.mc.otm.menu.CargoCrateMenu;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockEntityCargoCrate extends BlockEntityMattery {
	private static final TranslatableComponent NAME = new TranslatableComponent("block.overdrive_that_matters.cargo_crate");
	public int open_amount = 0;

	public void onPlayerOpen() {
		if (open_amount++ == 0 && level != null) {
			level.setBlock(getBlockPos(), getBlockState().setValue(BlockCargoCrate.IS_OPEN, true), Block.UPDATE_CLIENTS);
		}
	}

	public void onPlayerClose() {
		if (--open_amount == 0 && level != null) {
			level.setBlock(getBlockPos(), getBlockState().setValue(BlockCargoCrate.IS_OPEN, false), Block.UPDATE_CLIENTS);
		}
	}

	public final MatteryContainer container = new MatteryContainer(this::setChanged, 9 * 6);
	public final MatteryContainerHandler item_handler = container.handler();

	public BlockEntityCargoCrate(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.CARGO_CRATE, p_155229_, p_155230_);
	}

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		item_handler.invalidate();
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		item_handler.revive();
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == CapabilityItemHandler.ITEM_HANDLER_CAPABILITY)
			return item_handler.get().cast();

		return super.getCapability(cap, side);
	}

	@Override
	public void saveAdditional(CompoundTag nbt) {
		super.saveAdditional(nbt);
		nbt.put("slots", container.serializeNBT());
	}

	@Override
	public void load(CompoundTag nbt) {
		super.load(nbt);
		container.deserializeNBT(nbt.get("slots"));
	}

	@Override
	protected Component getDefaultDisplayName() {
		return NAME;
	}

	@Nullable
	@Override
	public AbstractContainerMenu createMenu(int containerID, Inventory inventory, Player ply) {
		return new CargoCrateMenu(containerID, inventory, this);
	}
}
