package ru.dbotthepony.mc.otm.capability;

import net.minecraftforge.common.capabilities.*;
import net.minecraftforge.energy.IEnergyStorage;
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability;
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive;
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.matter.IMatterTaskProvider;
import ru.dbotthepony.mc.otm.capability.matter.IPatternStorage;
import ru.dbotthepony.mc.otm.core.Fraction;
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode;
import ru.dbotthepony.mc.otm.graph.storage.IStorageGraphNode;

import java.math.MathContext;
import java.math.RoundingMode;

public class MatteryCapability {
	public static final Capability<IMatteryEnergyStorage> ENERGY = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IAndroidCapability> ANDROID = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterHandler> MATTER = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterGraphNode> MATTER_NODE = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IPatternStorage> PATTERN = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatterTaskProvider> TASK = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IMatteryDrive> DRIVE = CapabilityManager.get(new CapabilityToken<>() {});
	public static final Capability<IStorageGraphNode> STORAGE_NODE = CapabilityManager.get(new CapabilityToken<>() {});

	@SuppressWarnings("unused")
	public static void register(final RegisterCapabilitiesEvent event) {
		event.register(IMatteryEnergyStorage.class);
		event.register(IAndroidCapability.class);
		event.register(IMatterHandler.class);
		event.register(IPatternStorage.class);
		event.register(IMatterTaskProvider.class);
		event.register(IMatteryDrive.class);
		event.register(IStorageGraphNode.class);
		event.register(IMatterGraphNode.class);
	}

	public static final MathContext ROUND_RULES = new MathContext(32, RoundingMode.HALF_DOWN);

	public static final Fraction INT_MAX_VALUE = new Fraction(Integer.MAX_VALUE);
	public static final Fraction INT_MIN_VALUE = new Fraction(Integer.MIN_VALUE);

	public static final Fraction LONG_MAX_VALUE = new Fraction(Long.MAX_VALUE);
	public static final Fraction LONG_MIN_VALUE = new Fraction(Long.MIN_VALUE);

	public static Fraction drainFE(IEnergyStorage capability, Fraction howMuch, boolean simulate) {
		if (howMuch.compareTo(Fraction.ONE) < 0)
			return Fraction.ZERO;

		return new Fraction(capability.extractEnergy(howMuch.compareTo(INT_MAX_VALUE) > 0 ? Integer.MAX_VALUE : howMuch.toInt(), simulate));
	}

	public static Fraction floodFE(IEnergyStorage capability, Fraction howMuch, boolean simulate) {
		if (howMuch.compareTo(Fraction.ONE) < 0)
			return Fraction.ZERO;

		return new Fraction(capability.receiveEnergy(howMuch.compareTo(INT_MAX_VALUE) > 0 ? Integer.MAX_VALUE : howMuch.toInt(), simulate));
	}
}
