package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.EnumProperty
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityEnergyCounter

class BlockEnergyCounter : BlockMattery(), EntityBlock {
	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return BlockEntityEnergyCounter(blockPos, blockState)
	}

	override fun <T : BlockEntity?> getTicker(
		level: Level,
		blockState: BlockState,
		blockEntityType: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (level.isClientSide || blockEntityType !== Registry.BlockEntities.ENERGY_COUNTER)
			return null

		return BlockEntityTicker { _, _, _, tile -> if (tile is BlockEntityEnergyCounter) tile.tick() }
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		val inputDir =
			if (context.player != null && context.player!!.isCrouching) context.clickedFace.opposite else context.clickedFace

		val opposite = inputDir.opposite
		var dir: Direction? = null

		for (_dir in context.nearestLookingDirections) {
			if (_dir != inputDir && _dir != opposite) {
				dir = _dir.opposite
				break
			}
		}

		return defaultBlockState().setValue(INPUT_DIRECTION, inputDir).setValue(IF_DIRECTION, dir!!)
	}

	override fun createBlockStateDefinition(p_49915_: StateDefinition.Builder<Block, BlockState>) {
		super.createBlockStateDefinition(p_49915_)
		p_49915_.add(INPUT_DIRECTION, IF_DIRECTION)
	}

	override fun neighborChanged(
		state: BlockState,
		level: Level,
		pos: BlockPos,
		sender: Block,
		sender_pos: BlockPos,
		flag: Boolean
	) {
		super.neighborChanged(state, level, pos, sender, sender_pos, flag)

		if (!level.isClientSide) {
			(level.getBlockEntity(pos) as? BlockEntityEnergyCounter)?.checkSurroundings(level)
		}
	}

	companion object {
		@JvmField val INPUT_DIRECTION: EnumProperty<Direction> = EnumProperty.create("input", Direction::class.java)
		@JvmField val IF_DIRECTION: EnumProperty<Direction> = EnumProperty.create("if", Direction::class.java)
	}
}
