package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.graph.storage.BasicStorageGraphNode
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.menu.DriveRackMenu
import ru.dbotthepony.mc.otm.set
import ru.dbotthepony.mc.otm.graph.storage.StorageNetworkGraph
import ru.dbotthepony.mc.otm.storage.PoweredVirtualComponent

class BlockEntityDriveRack(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntityMatteryPowered(Registry.BlockEntities.DRIVE_RACK, p_155229_, p_155230_) {

	@JvmField
	val drives: MatteryContainer = object : MatteryContainer(this::setChanged, 4) {
		override fun setChanged(slot: Int, new_state: ItemStack, old_state: ItemStack) {
			super.setChanged(slot, new_state, old_state)

			old_state.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageIdentity()) {c -> PoweredVirtualComponent(c, energy)}.remove(it)
			}

			new_state.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageIdentity()) {c -> PoweredVirtualComponent(c, energy)}.add(it)
			}
		}
	}

	val cell = BasicStorageGraphNode()

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		nbt.ifHas("drives", CompoundTag::class.java) {
			drives.deserializeNBT(it)
		}
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["drives"] = drives.serializeNBT()
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			StorageNetworkGraph.discoverFull(this, cell.getAsStorageNode())
	}

	init {
		energy = MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, Fraction(80000))
	}

	fun tick() {
		batteryChargeLoop()
	}

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu? {
		return DriveRackMenu(containerID, inventory, this)
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		return if (cap === MatteryCapability.STORAGE_NODE) {
			cell.get().cast()
		} else super.getCapability(cap, side)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		cell.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		cell.revive()
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.drive_rack")
	}
}