package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityItemMonitor
import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.menu.data.INetworkedItemViewSupplier
import ru.dbotthepony.mc.otm.menu.data.NetworkedItemView
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper
import ru.dbotthepony.mc.otm.storage.PoweredVirtualComponent
import ru.dbotthepony.mc.otm.storage.VirtualComponent

class ItemMonitorMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: BlockEntityItemMonitor? = null
) : PoweredMatteryMenu(Registry.Menus.ITEM_MONITOR, p_38852_, inventory, tile), INetworkedItemViewSupplier {
	@JvmField
	val view = NetworkedItemView(inventory.player, this, tile == null)
	private val subscribed: VirtualComponent<ItemStackWrapper>?
	private val local: PoweredVirtualComponent<ItemStackWrapper>?

	override fun getNetworkedItemView(): NetworkedItemView {
		return view
	}

	init {
		if (tile != null) {
			subscribed = tile.cell.getStorageGraph()!!.getVirtualComponent(ItemStackWrapper::class.java)
			local = PoweredVirtualComponent(subscribed, tile.getCapability(MatteryCapability.ENERGY).resolve().get())
			view.setComponent(local)
		} else {
			subscribed = null
			local = null
		}

		addInventorySlots()
	}

	override fun removed(p_38940_: Player) {
		super.removed(p_38940_)
		view.removed()
		subscribed?.removeListenerAuto(local!!)
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		view.network()
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 1
	}

	override fun quickMoveStack(ply: Player, slot_index: Int): ItemStack {
		if (slot_index in inventorySlotIndexStart..inventorySlotIndexEnd) {

		}

		return super.quickMoveStack(ply, slot_index)
	}
}
